import uuid
import stix2
import os
import shutil

from uuid import UUID

from stix2 import Location
from stix2extensions._extensions import location_opencti_ExtensionDefinitionSMO

# create the directories

tmp_directories = [
    "tmp_object_store",
]

for directory in tmp_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

# define UUID for generating UUIDv5s -- use stix2extensions namespace as this is an SDO

namespace=UUID("1abb62b9-e513-5f55-8e73-8f6d7b55c237")

# define values that are recycled between objects

### dogesec-demo

created_by_ref="identity--" + str(uuid.uuid5(namespace, f"dogesec-demo"))
created="2020-01-01T00:00:00.000Z"
modified="2020-01-01T00:00:00.000Z"

# Create Location SDO object

### location--c06ee555-6c92-5d41-9e55-8abb490cb7c1

example_opencti_LocationSDO = Location(
                        id="location--"+ str(uuid.uuid5(namespace, f"A Demo Location")),
                        created_by_ref=created_by_ref,
                        created=created,
                        modified=modified,
                        name="Egypt",
                        country="EG",
                        x_opencti_aliases=[
                            "EGY",
                            "EG"
                        ],
                        x_opencti_location_type=[
                            "Country"
                        ],
                        latitude=26.820553,
                        longitude=30.802498,
                        region="northern-africa",
                        external_references=[
                            {
                              "source_name": "location2stix",
                              "external_id": "EG"
                            },
                            {
                              "source_name": "type",
                              "external_id": "country"
                            },
                            {
                              "source_name": "alpha-2",
                              "external_id": "EG"
                            },
                            {
                              "source_name": "alpha-3",
                              "external_id": "EGY"
                            },
                            {
                              "source_name": "iso_3166-2",
                              "external_id": "ISO 3166-2:EG"
                            },
                            {
                              "source_name": "country-code",
                              "external_id": "818"
                            }                        
                        ],
                        object_marking_refs=[
                            "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487", # this is TLP:CLEAR
                            "marking-definition--" + str(uuid.uuid5(namespace, f"stix2extensions")) # marking-definition--97ba4e8b-04f6-57e8-8f6e-3a0f0a7dc0fb
                        ],
                        extensions={
                            location_opencti_ExtensionDefinitionSMO.id: {
                                    "extension_type": "toplevel-property-extension"
                            }
                        }
                    )

# Write the objects to the filestore
## https://stix2.readthedocs.io/en/latest/guide/filesystem.html#FileSystemSource

### Creating FileSystemStore and adding MarkingDefinitionSMO for each directory

fs_directories = {
    "tmp_object_store": example_opencti_LocationSDO
}

for directory, location_sdo in fs_directories.items():
    fs_store = stix2.FileSystemStore(directory)
    fs_store.add([location_sdo])

# Now move those files into the standardised locations for easy download

final_directories = [
    "example_objects/sdos"
]

for directory in final_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

shutil.move("tmp_object_store/location/location--" + str(uuid.uuid5(namespace, f"A Demo Location")) + "/20200101000000000.json", "example_objects/properties/location--" + str(uuid.uuid5(namespace, f"A Demo Location")) + ".json")

shutil.rmtree("tmp_object_store")


from functools import lru_cache
import logging
import uuid
import requests
from stix2 import Relationship, Identity
from .._extensions import DOGESEC_IDENTITY_REF

from ..bank_card import BankCard

IDENTITY_NS = uuid.UUID("d287a5a4-facc-5254-9563-9e92e3e729ac")
OASIS_NS    = uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7")

@lru_cache(maxsize=4096)
def get_bin_data(card_number, api_key):
    try:
        bin_number = card_number[:6]
        url = f"https://bin-ip-checker.p.rapidapi.com/?bin={bin_number}"
        headers = {
            'Content-Type': 'application/json',
            'x-rapidapi-host': 'bin-ip-checker.p.rapidapi.com',
            'x-rapidapi-key': api_key
        }
        logging.debug(f"Requesting BIN data for card number: {card_number}")
        response = requests.post(url, headers=headers, json={"bin": bin_number}, timeout=10)
        response.raise_for_status()
        logging.debug(f"Received response: {response.json()}")
        return response.json()
    except requests.RequestException as e:
        logging.error(f"Error fetching BIN data for {card_number}: {e}")
        return None

def create_identity(bin_data):
    issuer = bin_data['BIN']['issuer']
    country = bin_data['BIN']['country']
    name = f"{issuer['name']} ({country['alpha2']})"
    identity_id = f"identity--{str(uuid.uuid5(IDENTITY_NS, name))}"

    if not issuer['name']:
        return Identity(
            type="identity",
            spec_version="2.1",
            id="identity--643246fc-9204-5b4b-976d-2e605b355c24",
            created_by_ref=DOGESEC_IDENTITY_REF,
            created="2020-01-01T00:00:00.000Z",
            modified="2020-01-01T00:00:00.000Z",
            name="Unknown Bank",
            identity_class="organization",
            sectors=[
                "financial-services"
            ],
            object_marking_refs=[
                "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487",
                "marking-definition--d287a5a4-facc-5254-9563-9e92e3e729ac"
            ]
        )


    return Identity(
        id=identity_id,
        name=name,
        created="2020-01-01T00:00:00.000Z",
        modified="2020-01-01T00:00:00.000Z",
        identity_class="organization",
        sectors=["financial-services"],
        contact_information=f"* Bank URL: {issuer['website']},\n* Bank Phone: {issuer['phone']}",
        created_by_ref=DOGESEC_IDENTITY_REF,
        object_marking_refs=[
            "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487",
            "marking-definition--d287a5a4-facc-5254-9563-9e92e3e729ac"
        ]
    )

def create_credit_card_stix(card_data, bin_data):
    card_id = f"bank-card--{str(uuid.uuid5(OASIS_NS, card_data['card_number']))}"

    credit_card_data = {
            'type': 'bank-card',
            'spec_version': '2.1',
            'id': card_id,
            'number': card_data['card_number'],
    }
    if bin_data:
        credit_card_data.update(
            format=bin_data['BIN']['type'],
            scheme=bin_data['BIN']['scheme'],
            brand=bin_data['BIN']['brand'],
            currency=bin_data['BIN']['currency'],
            #
            level=bin_data['BIN']['level'],
            is_commercial=bin_data['BIN']['is_commercial'] == 'true',
            is_prepaid=bin_data['BIN']['is_prepaid'] == 'true',
        )
        
    # Add optional fields if they are present and not empty
    optional_fields = ['card_holder_name', 'card_valid_date', 'card_expiry_date', 'card_security_code']
    field_mapping = {
        'card_holder_name': 'holder_name',
        'card_valid_date': 'valid_from',
        'card_expiry_date': 'valid_to',
        'card_security_code': 'security_code'
    }
        
    for field in optional_fields:
        if card_data.get(field):
            credit_card_data[field_mapping[field]] = card_data[field]
    
    return credit_card_data


def create_objects(card_data, api_key):
    bin_data = get_bin_data(card_data['card_number'], api_key)
    card = create_credit_card_stix(card_data, bin_data)
    retval = []
    if bin_data and bin_data['BIN']['valid']:
        identity = create_identity(bin_data)
        retval.append(identity)
        card.update(issuer_ref=identity.id)
    retval.insert(0, BankCard(**card))
    return retval

if __name__ == '__main__':
    import os
    from pprint import pprint

    pprint(create_objects({'card_number': "559666123232123112312312332"}, os.getenv("BIN_LIST_API_KEY")))
    pprint(create_objects({'card_number': "410540"}, os.getenv("BIN_LIST_API_KEY")))




from stix2 import CustomObservable
from stix2.properties import (
    ExtensionsProperty, ReferenceProperty,
    IDProperty, ListProperty, StringProperty,
    TimestampProperty, TypeProperty, DictionaryProperty
)
from ._extensions import cryptocurrency_transaction_ExtensionDefinitionSMO

_type = 'cryptocurrency-transaction'
@CustomObservable('cryptocurrency-transaction', [
    ('type', TypeProperty(_type, spec_version='2.1')),
    ('spec_version', StringProperty(fixed='2.1')),
    ('id', IDProperty(_type, spec_version='2.1')),
    ('symbol', StringProperty(required=True)),
    ('hash', StringProperty(required=True)),
    ('fee', StringProperty()),
    ('execution_time', TimestampProperty()),
    ('input', ListProperty(DictionaryProperty())),
    ('output', ListProperty(DictionaryProperty())),
], extension_name=cryptocurrency_transaction_ExtensionDefinitionSMO.id, id_contrib_props=['hash', 'symbol'])
class CryptocurrencyTransaction(object):
    pass



import stix2
from pathlib import Path
class Generator:
    def __init__(self, path) -> None:
        self.path = Path(path)
        self.items : dict[str, stix2.observables._Observable] = dict()
        self.path.mkdir(parents=True, exist_ok=True)
        # self.type = type
    def add_item(self, name, item):
        self.items[name] = item
        # self.fs.add(item)

    def save_all(self):
        for name, item in self.items.items():
            path = self.path/f"{name}.json"
            path.write_text(item.serialize(pretty=True))
            print(f"{name} => {path}")



from stix2 import CustomObservable
from stix2.properties import (
    ExtensionsProperty, ReferenceProperty,
    IDProperty, ListProperty, StringProperty,
    TypeProperty,
)
from ._extensions import user_agent_ExtensionDefinitionSMO

_type = 'user-agent'
@CustomObservable('user-agent', [
    ('type', TypeProperty(_type, spec_version='2.1')),
    ('spec_version', StringProperty(fixed='2.1')),
    ('id', IDProperty(_type, spec_version='2.1')),
    ('string', StringProperty(required=True)),
    # ('extensions', ExtensionsProperty(spec_version='2.1'))
], extension_name=user_agent_ExtensionDefinitionSMO.id, id_contrib_props=['string'])
class UserAgent(object):
    pass




from stix2 import CustomObservable
from stix2.properties import (
    ExtensionsProperty, ReferenceProperty,
    IDProperty, ListProperty, StringProperty,
    TypeProperty,
)

from ._extensions import bank_account_ExtensionDefinitionSMO

_type = 'bank-account'
@CustomObservable('bank-account', [
    ('type', TypeProperty(_type, spec_version='2.1')),
    ('spec_version', StringProperty(fixed='2.1')),
    ('id', IDProperty(_type, spec_version='2.1')),
    ('bank', StringProperty()),
    ('country', StringProperty()),
    ('currency', StringProperty()),
    ('holder_name', StringProperty()),
    ('iban_number', StringProperty(required=True)),
    ('swift_code', StringProperty()),
    # ('extensions', ExtensionsProperty(spec_version='2.1'))
], extension_name=bank_account_ExtensionDefinitionSMO.id, id_contrib_props=['iban_number'])
class BankAccount(object):
    pass


import uuid
import requests
from stix2 import parse, Identity, Relationship
import logging

def load_stix_object_from_url(url):
    try:
        response = requests.get(url)
        response.raise_for_status()  # Raise an HTTPError for bad responses
        return parse(response.text)
    except requests.exceptions.RequestException as e:
        logging.error(f"Error loading JSON from {url}: {e}")
        raise




from stix2 import CustomObservable
from stix2.properties import (
    ExtensionsProperty, ReferenceProperty,
    IDProperty, ListProperty, StringProperty,
    TypeProperty,
)
from ._extensions import phone_number_ExtensionDefinitionSMO

_type = 'phone-number'
@CustomObservable('phone-number', [
    ('type', TypeProperty(_type, spec_version='2.1')),
    ('spec_version', StringProperty(fixed='2.1')),
    ('id', IDProperty(_type, spec_version='2.1')),
    ('number', StringProperty(required=True)),
    ('country', StringProperty()),
    ('connection', StringProperty()),
    ('provider', StringProperty()),
    # ('extensions', ExtensionsProperty(spec_version='2.1'))
], extension_name=phone_number_ExtensionDefinitionSMO.id, id_contrib_props=['number'])
class Phonenumber(object):
    pass




from stix2 import CustomObservable
from stix2.properties import (
    ExtensionsProperty, ReferenceProperty,
    IDProperty, ListProperty, StringProperty,
    TypeProperty,
)
from ._extensions import cryptocurrency_wallet_ExtensionDefinitionSMO
_type = 'cryptocurrency-wallet'
@CustomObservable('cryptocurrency-wallet', [
    ('type', TypeProperty(_type, spec_version='2.1')),
    ('spec_version', StringProperty(fixed='2.1')),
    ('id', IDProperty(_type, spec_version='2.1')),
    ('address', StringProperty(required=True)),
    ('holder_ref', ReferenceProperty(valid_types='identity', spec_version='2.1')),
    # ('extensions', ExtensionsProperty(spec_version='2.1'))
], extension_name=cryptocurrency_wallet_ExtensionDefinitionSMO.id,id_contrib_props=['address'])
class CryptocurrencyWallet(object):
    pass



import uuid
import stix2
import os
import shutil

from uuid import UUID
from stix2extensions.bank_card import BankCard
# create the directories

tmp_directories = [
    "tmp_object_store",
]

for directory in tmp_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

# define UUID for generating UUIDv5s -- this is the OASIS namespace for SCOs https://github.com/oasis-open/cti-python-stix2/blob/master/stix2/base.py#L29

namespace=UUID("00abedb4-aa42-466c-9c01-fed23315a9b7")

# Create bank-card SCO

example_bankCardSCO = BankCard(
                    id="bank-card--"+ str(uuid.uuid5(namespace, f"4242424242424242")), # bank-card--9ce64b19-095d-5187-a56b-79a82ae4066f
                    format="credit",
                    number="4242424242424242",
                    scheme="VISA",
                    brand="VISA",
                    currency="GBP",
                    issuer_ref="identity--9bef1584-289b-41fe-81b4-a5d72b7746d6",
                    holder_ref="identity--9fcd0db9-8115-4e33-ae00-674806395cf1",
                    valid_from="01/99",
                    valid_to="01/00",
                    security_code="999",
                    level="CLASSIC",
                    is_commercial=False,
                    is_prepaid=False
                    )

# Write the objects to the filestore
## https://stix2.readthedocs.io/en/latest/guide/filesystem.html#FileSystemSource

### Creating FileSystemStore and adding MarkingDefinitionSMO for each directory

fs_directories = {
    "tmp_object_store": example_bankCardSCO
}

for directory, bankcard_sco in fs_directories.items():
    fs_store = stix2.FileSystemStore(directory)
    fs_store.add([bankcard_sco])

# Now move those files into the standardised locations for easy download

final_directories = [
    "example_objects/scos"
]

for directory in final_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

shutil.move("tmp_object_store/bank-card/bank-card--" + str(uuid.uuid5(namespace, f"4242424242424242")) + ".json", "example_objects/scos/bank-card--" + str(uuid.uuid5(namespace, f"4242424242424242")) + ".json")

shutil.rmtree("tmp_object_store")


import uuid
import hashlib
import stix2
import os
import json
import sys

from stix2 import Bundle
from stix2.base import STIXJSONEncoder
from uuid import UUID

from stix2extensions._extensions import (
    weakness_ExtensionDefinitionSMO,
    exploit_ExtensionDefinitionSMO,
    bank_account_ExtensionDefinitionSMO,
    bank_card_ExtensionDefinitionSMO,
    cryptocurrency_transaction_ExtensionDefinitionSMO,
    cryptocurrency_wallet_ExtensionDefinitionSMO,
    data_source_ExtensionDefinitionSMO,
    phone_number_ExtensionDefinitionSMO,
    user_agent_ExtensionDefinitionSMO,
    vulnerability_scoring_ExtensionDefinitionSMO,
    vulnerability_opencti_ExtensionDefinitionSMO,
    indicator_vulnerable_cpes_ExtensionDefinitionSMO,
    report_epss_scoring_ExtensionDefinitionSMO,
    software_cpe_properties_ExtensionDefinitionSMO,
    location_opencti_ExtensionDefinitionSMO
)

sys.path.append('generators')

from utils import Generator

# Namespace UUID for generating UUIDv5
namespace = UUID("00abedb4-aa42-466c-9c01-fed23315a9b7")

# Create and save the objects using the Generator
if __name__ == '__main__':
    generator_sdos = Generator("extension-definitions/sdos")
    generator_sdos.add_item("weakness", weakness_ExtensionDefinitionSMO)
    generator_sdos.add_item("exploit", exploit_ExtensionDefinitionSMO)
    
    generator_scos = Generator("extension-definitions/scos")
    generator_scos.add_item("bank-account", bank_account_ExtensionDefinitionSMO)
    generator_scos.add_item("bank-card", bank_card_ExtensionDefinitionSMO)
    generator_scos.add_item("cryptocurrency-transaction", cryptocurrency_transaction_ExtensionDefinitionSMO)
    generator_scos.add_item("cryptocurrency-wallet", cryptocurrency_wallet_ExtensionDefinitionSMO)
    generator_scos.add_item("data-source", data_source_ExtensionDefinitionSMO)
    generator_scos.add_item("phone-number", phone_number_ExtensionDefinitionSMO)
    generator_scos.add_item("user-agent", user_agent_ExtensionDefinitionSMO)
    
    generator_properties = Generator("extension-definitions/properties")
    generator_properties.add_item("vulnerability-scoring", vulnerability_scoring_ExtensionDefinitionSMO)
    generator_properties.add_item("vulnerability-opencti", vulnerability_opencti_ExtensionDefinitionSMO)
    generator_properties.add_item("indicator-vulnerable-cpes", indicator_vulnerable_cpes_ExtensionDefinitionSMO)
    generator_properties.add_item("report-epss-scoring", report_epss_scoring_ExtensionDefinitionSMO)
    generator_properties.add_item("software-cpe-properties", software_cpe_properties_ExtensionDefinitionSMO)
    generator_properties.add_item("location-opencti", location_opencti_ExtensionDefinitionSMO)
    

    # Save all items
    generator_sdos.save_all()
    generator_scos.save_all()
    generator_properties.save_all()

    # Combine all objects from all generators into one bundle
    all_objects = list(generator_sdos.items.values()) + list(generator_scos.items.values()) + list(generator_properties.items.values())

    # Sort the objects by their ID to ensure consistent order
    all_objects = sorted(all_objects, key=lambda x: x['id'])

    # Create the STIX bundle
    bundle_of_all_objects = {
        "type": "bundle",
        "objects": all_objects
    }

    # Serialize the objects for MD5 hash calculation (sort_keys ensures consistency)
    bundle_objects_json_str = json.dumps(bundle_of_all_objects['objects'], cls=STIXJSONEncoder, sort_keys=True)

    # Generate the MD5 hash of the objects
    md5_hash = hashlib.md5(bundle_objects_json_str.encode('utf-8')).hexdigest()

    # Generate UUIDv5 for the bundle using the namespace and MD5 hash
    bundle_id = "bundle--" + str(uuid.uuid5(namespace, md5_hash))

    # Create the final bundle with the calculated ID
    final_bundle = Bundle(id=bundle_id, objects=all_objects, allow_custom=True)

    # Save the bundle to a file
    with open("extension-definitions/extension-definition-bundle.json", "w") as bundle_file:
        json.dump(final_bundle, bundle_file, cls=STIXJSONEncoder, indent=4)

    # Print the serialized bundle
    print(final_bundle.serialize(pretty=True))



import uuid
import stix2
import os
import shutil

from uuid import UUID

from stix2 import Vulnerability
from stix2extensions._extensions import vulnerability_opencti_ExtensionDefinitionSMO

# create the directories

tmp_directories = [
    "tmp_object_store",
]

for directory in tmp_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

# define UUID for generating UUIDv5s -- use stix2extensions namespace as this is an SDO

namespace=UUID("1abb62b9-e513-5f55-8e73-8f6d7b55c237")

# define values that are recycled between objects

### dogesec-demo

created_by_ref="identity--" + str(uuid.uuid5(namespace, f"dogesec-demo"))
created="2020-01-01T00:00:00.000Z"
modified="2020-01-01T00:00:00.000Z"

# Create Vulnerability SDO object

### vulnerability--132b38d2-c0d2-5c11-967f-9959a7099a2f

example_opencti_VulnerabilitySDO = Vulnerability(
                        id="vulnerability--"+ str(uuid.uuid5(namespace, f"Another demo Vulnerability")),
                        created_by_ref=created_by_ref,
                        created=created,
                        modified=modified,
                        name="CVE-XXX-XXXX",
                        external_references=[
                            {
                                "source_name": "cve",
                                "url": "https://nvd.nist.gov/vuln/detail/CVE-XXX-XXXX",
                                "external_id": "CVE-XXX-XXXX"
                            }
                        ],
                        object_marking_refs=[
                            "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487", # this is TLP:CLEAR
                            "marking-definition--" + str(uuid.uuid5(namespace, f"stix2extensions")) # marking-definition--97ba4e8b-04f6-57e8-8f6e-3a0f0a7dc0fb
                        ],
                        x_opencti_cvss_v2_base_score=4.8,
                        x_opencti_cvss_v2_vector_string="AV:N/AC:L/Au:N/C:P/I:P/A:P",
                        x_opencti_cvss_base_score=4.8,
                        x_opencti_cvss_vector_string="CVSS:3.1/AV:N/AC:L/PR:H/UI:R/S:C/C:L/I:L/A:N",
                        extensions={
                            vulnerability_opencti_ExtensionDefinitionSMO.id: {
                                    "extension_type": "toplevel-property-extension"
                            }
                        }
                    )

# Write the objects to the filestore
## https://stix2.readthedocs.io/en/latest/guide/filesystem.html#FileSystemSource

### Creating FileSystemStore and adding MarkingDefinitionSMO for each directory

fs_directories = {
    "tmp_object_store": example_opencti_VulnerabilitySDO
}

for directory, vulnerability_sdo in fs_directories.items():
    fs_store = stix2.FileSystemStore(directory)
    fs_store.add([vulnerability_sdo])

# Now move those files into the standardised locations for easy download

final_directories = [
    "example_objects/sdos"
]

for directory in final_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

shutil.move("tmp_object_store/vulnerability/vulnerability--" + str(uuid.uuid5(namespace, f"Another demo Vulnerability")) + "/20200101000000000.json", "example_objects/properties/vulnerability--" + str(uuid.uuid5(namespace, f"Another demo Vulnerability")) + ".json")

shutil.rmtree("tmp_object_store")


import uuid
import stix2
import os
import shutil

from uuid import UUID

from stix2extensions.exploit import Exploit
# create the directories

tmp_directories = [
    "tmp_object_store",
]

for directory in tmp_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

# define UUID for generating UUIDv5s -- use stix2extensions namespace as this is an SDO

namespace=UUID("1abb62b9-e513-5f55-8e73-8f6d7b55c237")

# define values that are recycled between objects

### dogesec-demo

created_by_ref="identity--" + str(uuid.uuid5(namespace, f"dogesec-demo"))
created="2020-01-01T00:00:00.000Z"
modified="2020-01-01T00:00:00.000Z"

# Create Exploit SDO object

### exploit--519a9c84-3c54-5698-bd9a-99bd01f003d1

example_ExploitSDO = Exploit(
                        id="exploit--"+ str(uuid.uuid5(namespace, f"A demo exploit")),
                        created_by_ref=created_by_ref,
                        created=created,
                        modified=modified,
                        name="Exploit Demo",
                        description="A demo exploit",
                        vulnerability_ref="vulnerability--c5dcf0f3-4afe-4458-b79b-0b3e671db8ed",
                        exploit_type="client-side",
                        proof_of_concept="git@github.com:dogesec/CVE-0000-00000-Proof-Of-Concept.git",
                        external_references=[
                            {
                                "source_name": "cve",
                                "external_id": "CVE-0000-00000",
                                "url": "https://nvd.nist.gov/vuln/detail/CVE-0000-00000"
                            }
                        ],
                        object_marking_refs=[
                            "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487", # this is TLP:CLEAR
                            "marking-definition--" + str(uuid.uuid5(namespace, f"stix2extensions")) # marking-definition--97ba4e8b-04f6-57e8-8f6e-3a0f0a7dc0fb
                        ]
                    )

# Write the objects to the filestore
## https://stix2.readthedocs.io/en/latest/guide/filesystem.html#FileSystemSource

### Creating FileSystemStore and adding MarkingDefinitionSMO for each directory

fs_directories = {
    "tmp_object_store": example_ExploitSDO
}

for directory, exploit_sdo in fs_directories.items():
    fs_store = stix2.FileSystemStore(directory)
    fs_store.add([exploit_sdo])

# Now move those files into the standardised locations for easy download

final_directories = [
    "example_objects/sdos"
]

for directory in final_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

shutil.move("tmp_object_store/exploit/exploit--" + str(uuid.uuid5(namespace, f"A demo exploit")) + "/20200101000000000.json", "example_objects/sdos/exploit--" + str(uuid.uuid5(namespace, f"A demo exploit")) + ".json")

shutil.rmtree("tmp_object_store")



from stix2 import CustomObject
from stix2.properties import (
    ReferenceProperty,
    IDProperty, ListProperty, StringProperty,
    TimestampProperty, TypeProperty, BooleanProperty,
)
from stix2.v21.common import (
    ExternalReference,
)
from stix2.utils import NOW
from ._extensions import exploit_ExtensionDefinitionSMO

_type = 'exploit'
@CustomObject('exploit', [
    ('type', TypeProperty(_type, spec_version='2.1')),
    ('spec_version', StringProperty(fixed='2.1')),
    ('id', IDProperty(_type, spec_version='2.1')),
    ('created_by_ref', ReferenceProperty(valid_types='identity', spec_version='2.1')),
    ('created', TimestampProperty(default=lambda: NOW, precision='millisecond', precision_constraint='min')),
    ('modified', TimestampProperty(default=lambda: NOW, precision='millisecond', precision_constraint='min')),
    ('name', StringProperty(required=True)),
    ('description', StringProperty()),
    ('revoked', BooleanProperty(default=lambda: False)),
    ('vulnerability_ref', ReferenceProperty(valid_types='vulnerability', spec_version='2.1')),
    ('exploit_type', StringProperty()),
    ('proof_of_concept', StringProperty()),
    ('external_references', ListProperty(ExternalReference)),
    ('object_marking_refs', ListProperty(ReferenceProperty(valid_types='marking-definition', spec_version='2.1'))),
    # ('extensions', ExtensionsProperty(spec_version='2.1'))
], extension_name=exploit_ExtensionDefinitionSMO.id)
class Exploit(object):
    pass


import os
import shutil
import uuid
from uuid import UUID

import stix2
from stix2extensions.bank_account import BankAccount

# create the directories

tmp_directories = [
    "tmp_object_store",
]

for directory in tmp_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

# define UUID for generating UUIDv5s -- this is the OASIS namespace for SCOs https://github.com/oasis-open/cti-python-stix2/blob/master/stix2/base.py#L29

namespace=UUID("00abedb4-aa42-466c-9c01-fed23315a9b7")

# Create bank account SCO

example_bankAccountSCO = BankAccount(
                    id="bank-account--"+ str(uuid.uuid5(namespace, f"GB33BUKB20201555555555")), # bank-account--fb76b7b8-8701-5d8b-a143-0283847f6638
                    bank="Big Bank",
                    country="GBR",
                    currency="GBP",
                    holder_name="DOGESEC",
                    iban_number="GB33BUKB20201555555555",
                    swift_code="DEMOGB22XXX",
                    )

# Write the objects to the filestore
## https://stix2.readthedocs.io/en/latest/guide/filesystem.html#FileSystemSource

### Creating FileSystemStore and adding MarkingDefinitionSMO for each directory

fs_directories = {
    "tmp_object_store": example_bankAccountSCO
}

for directory, bankaccount_sco in fs_directories.items():
    fs_store = stix2.FileSystemStore(directory)
    fs_store.add([bankaccount_sco])

# Now move those files into the standardised locations for easy download

final_directories = [
    "example_objects/scos"
]

for directory in final_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

shutil.move("tmp_object_store/bank-account/bank-account--" + str(uuid.uuid5(namespace, f"GB33BUKB20201555555555")) + ".json", "example_objects/scos/bank-account--" + str(uuid.uuid5(namespace, f"GB33BUKB20201555555555")) + ".json")

shutil.rmtree("tmp_object_store")



from stix2 import CustomObservable
from stix2.properties import (
    ExtensionsProperty,
    ReferenceProperty,
    IDProperty,
    ListProperty,
    StringProperty,
    BooleanProperty,
    TypeProperty,
)
from ._extensions import data_source_ExtensionDefinitionSMO

_type = "data-source"


@CustomObservable(
    "data-source",
    [
        ("type", TypeProperty(_type, spec_version="2.1")),
        ("spec_version", StringProperty(fixed="2.1")),
        ("id", IDProperty(_type, spec_version="2.1")),
        ("category", StringProperty()),
        ("product", StringProperty()),
        ("service", StringProperty()),
        ("definition", StringProperty()),
    ],
    extension_name=data_source_ExtensionDefinitionSMO.id,
    id_contrib_props=["category", "product", "service"],
)
class DataSource(object):
    pass



from dataclasses import dataclass
from datetime import UTC, datetime
from itertools import chain
import time
from typing import Union
import uuid
import requests
from ..cryptocurrency_transaction import CryptocurrencyTransaction
from ..cryptocurrency_wallet import CryptocurrencyWallet
from .._extensions import (
    cryptocurrency_transaction_ExtensionDefinitionSMO,
    cryptocurrency_wallet_ExtensionDefinitionSMO,
)

# this is the oasis uuid
WALLET_NAMESPACE_UUID = uuid.UUID("00abedb4-aa42-466c-9c01-fed23315a9b7")

@dataclass
class TxnData:
    block_id: str = ""
    execution_time: str = ""
    fee: str = ""
    inputs: list[tuple[str, str]] = ()
    outputs: list[tuple[str, str]] = ()
    hash: str = ""

@dataclass
class WalletData:
    address: str = ""
    transactions: list[str] = ()


class Crypto2Stix:
    symbol = None
    processed_wallets = set()

    def get_transaction_object(self, hash) -> CryptocurrencyTransaction:
        raise NotImplementedError("should be implemented in subclass")

    def get_transaction_data(self, tx_hash) -> TxnData:
        raise NotImplementedError("should be implemented in subclass")

    def get_wallet_data(self, addr) -> WalletData:
        raise NotImplementedError("should be implemented in subclass")

    def create_transaction_object(self, tx_data: TxnData):
        transaction_object = CryptocurrencyTransaction(
            type="cryptocurrency-transaction",
            spec_version="2.1",
            symbol=self.symbol,
            hash=tx_data.hash,
            # block_id=tx_data.block_id,
            fee=tx_data.fee,
            execution_time=tx_data.execution_time,
            input=[
                {
                    "address_ref": f"cryptocurrency-wallet--{str(uuid.uuid5(WALLET_NAMESPACE_UUID, addr))}",
                    "amount": amount,
                }
                for addr, amount in tx_data.inputs
            ],
            output=[
                {
                    "address_ref": f"cryptocurrency-wallet--{str(uuid.uuid5(WALLET_NAMESPACE_UUID, addr))}",
                    "amount": amount,
                }
                for addr, amount in tx_data.outputs
            ],
        )
        return transaction_object

    def create_wallet_object(self, addr):
        # wallet = self.get_wallet_data(addr)
        return CryptocurrencyWallet(
            type="cryptocurrency-wallet",
            spec_version="2.1",
            id=self.get_wallet_id(addr),
            address=addr,
            extensions={
                cryptocurrency_wallet_ExtensionDefinitionSMO.id: {
                    "extension_type": "new-sco"
                }
            },
        )

    @staticmethod
    def get_wallet_id(wallet_addr):
        return f"cryptocurrency-wallet--{str(uuid.uuid5(WALLET_NAMESPACE_UUID, wallet_addr))}"

    @staticmethod
    def get_txn_id(txn_hash):
        return f"cryptocurrency-transaction--{str(uuid.uuid5(WALLET_NAMESPACE_UUID, txn_hash))}"
    
    def process_transaction(self, txn: Union[str, TxnData]):
        tx_data = self.get_transaction_data(txn)
        objects = [self.create_transaction_object(tx_data)]
        for addr, _ in chain(tx_data.inputs, tx_data.outputs):
            if addr in self.processed_wallets:
                continue
            objects.append(
                self.create_wallet_object(addr)
            )
            self.processed_wallets.add(addr)
        return objects
    
    def process_wallet(self, addr: str, transactions_only=True, wallet_only=False):
        objects = []
        wallet = WalletData(address=addr)
        if not wallet_only:
            wallet = self.get_wallet_data(addr)
        objects.append(self.create_wallet_object(wallet.address))
        self.processed_wallets.add(addr)
        for txn_hash in wallet.transactions:
            if transactions_only:
                txn = self.get_transaction_data(txn_hash)
                objects.append(self.create_transaction_object(txn))
            else:
                objects.extend(self.process_transaction(txn_hash))
        return objects
            

class BTC2Stix(Crypto2Stix):
    symbol = "BTC"
    DIVIDER = 100000000

    def get_transaction_object(self, hash):
        url = f"https://btcscan.org/api/tx/{hash}"
        return self.make_request(url)

    def get_transaction_data(self, txn):
        if isinstance(txn, str):
            txn = self.get_transaction_object(txn)
        if isinstance(txn, TxnData):
            return txn

        inputs = []
        for inp in txn.get("vin", []):
            prevout = self.parse_tx_address_and_value(inp.get("prevout", {}))
            if prevout:
                inputs.append(prevout)

        outputs = []
        for out in txn.get("vout", []):
            out_parsed = self.parse_tx_address_and_value(out)
            if out_parsed:
                outputs.append(out_parsed)

        status = txn.get("status", {})
        block_id = str(status.get("block_height")) if status.get("block_height") else None
        block_time = status.get("block_time")
        execution_time = datetime.fromtimestamp(block_time, tz=UTC) if block_time else None

        return TxnData(
            block_id=block_id,
            execution_time=execution_time,
            fee=str(txn["fee"] / self.DIVIDER) if txn.get("fee") else "0",
            inputs=inputs,
            outputs=outputs,
            hash=txn.get("txid"),
        )
    

    def get_wallet_data(self, wallet_address):
        return WalletData(address=wallet_address, transactions=self.get_transactions_by_address(wallet_address))
    
    def get_transactions_by_address(self, address):
        all_txns = []
        last_seen_txid = None

        while True:
            url = f"https://btcscan.org/api/address/{address}/txs/chain"
            if last_seen_txid:
                url += f"/{last_seen_txid}"
            batch = self.make_request(url)
            if not batch:
                break  # No more transactions
            all_txns.extend(batch)
            last_seen_txid = batch[-1]["txid"]
        return all_txns
    
    @classmethod
    def parse_tx_address_and_value(cls, out: dict):
        if not out:
            return
        addr = out.get("scriptpubkey_address")
        value = out.get("value")
        if addr and value is not None:
            return (addr, value / cls.DIVIDER)
        
    def make_request(self, url):
        time.sleep(1)
        for attempt in range(5):
            response = requests.get(url)
            if response.status_code == 429:
                time.sleep(60)
                continue
            response.raise_for_status()
            return response.json()

        raise Exception("Failed to fetch transaction data after 10 retries due to rate limits.")



from .bank_account import BankAccount
from .bank_card import BankCard
from .cryptocurrency_transaction import CryptocurrencyTransaction
from .cryptocurrency_wallet import CryptocurrencyWallet
from .phone_number import Phonenumber
from .user_agent import UserAgent
from .weakness import Weakness
from .attack_action import AttackAction, AttackFlow
from .exploit import Exploit


from datetime import UTC, datetime, timezone
import uuid
import stix2
import os
import shutil
import json

from stix2 import Bundle
from stix2.base import STIXJSONEncoder
from stix2 import ExtensionDefinition
from stix2 import FileSystemStore
from uuid import UUID

# define UUID for generating UUIDv5s

namespace=UUID("1abb62b9-e513-5f55-8e73-8f6d7b55c237")

# define values that are recycled between objects

### dogesec

DOGESEC_IDENTITY_REF="identity--" + str(uuid.uuid5(namespace, f"dogesec"))
created="2020-01-01T00:00:00.000Z"
modified="2020-01-01T00:00:00.000Z"
schema_base="https://raw.githubusercontent.com/muchdogesec/stix2extensions/main/schemas/"

### mitre TLP:CLEAR and stix4doge

object_marking_refs=[
    "marking-definition--94868c89-83c2-464b-929b-a1a8aa3c8487", # this is TLP:CLEAR
    "marking-definition--" + str(uuid.uuid5(namespace, f"stix2extensions")) # 
]

# Create New Extension Defintion SMO
## https://stix2.readthedocs.io/en/latest/api/v21/stix2.v21.common.html

### Phone number SMO

phone_number_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"phone-number")), # extension-definition--14a97ee2-e666-5ada-a6bd-b7177f79e211
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2020, 1, 1, tzinfo=UTC),
                        name="Phone Number",
                        description="This extension creates a new SCO that can be used to represent phone numbers.",
                        schema=schema_base+"scos/phone-number.json",
                        version="1.0",
                        extension_types=[
                            "new-sco"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Cryptocurrency Transaction SMO

cryptocurrency_transaction_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"cryptocurrency-transaction")), # extension-definition--151d042d-4dcf-5e44-843f-1024440318e5
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2020, 1, 1, tzinfo=UTC),
                        name="Cryptocurrency Transaction",
                        description="This extension creates a new SCO that can be used to represent cryptocurrency transactions.",
                        schema=schema_base+"scos/cryptocurrency-transaction.json",
                        version="1.0",
                        extension_types=[
                            "new-sco"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Data Source SMO

data_source_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"data-source")), # extension-definition--afeeb724-bce2-575e-af3d-d705842ea84b
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=modified,
                        name="Data Source",
                        description="This extension creates a new SCO that can be used to represent data sources. Very similar to x-mitre-data-source objects used in ATT&CK.",
                        schema=schema_base+"scos/data-source.json",
                        version="1.0",
                        extension_types=[
                            "new-sco"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Weakness SMO

weakness_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"weakness")), # extension-definition--31725edc-7d81-5db7-908a-9134f322284a
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2020, 1, 1, tzinfo=UTC),
                        name="Weakness",
                        description="This extension creates a new SDO that can be used to represent weaknesses (for CWEs).",
                        schema=schema_base+"sdos/weakness.json",
                        version="1.0",
                        extension_types=[
                            "new-sdo"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Exploit SMO

exploit_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"exploit")), # extension-definition--5a047f57-0149-59b6-a079-e2d7c7ac799a
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2020, 1, 1, tzinfo=UTC),
                        name="Exploit",
                        description="This extension creates a new SDO that can be used to represent exploits (of CVEs).",
                        schema=schema_base+"sdos/exploit.json",
                        version="1.0",
                        extension_types=[
                            "new-sdo"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Bank Card SMO

bank_card_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"bank-card")), # extension-definition--7922f91a-ee77-58a5-8217-321ce6a2d6e0
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2020, 1, 1, tzinfo=UTC),
                        name="Bank Card",
                        description="This extension creates a new SCO that can be used to represent bank cards.",
                        schema=schema_base+"scos/bank-card.json",
                        version="1.0",
                        extension_types=[
                            "new-sco"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### User Agent SMO

user_agent_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"user-agent")), # extension-definition--7ca5afee-0e4e-5813-b643-de51538658cc
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2020, 1, 1, tzinfo=UTC),
                        name="User Agent",
                        description="This extension creates a new SCO that can be used to represent user agents used in HTTP request. It is designed to be used when the Network Traffic SCO with HTTP request extension cannot be used due to lack of request information needed for the required properties.",
                        schema=schema_base+"scos/user-agent.json",
                        version="1.0",
                        extension_types=[
                            "new-sco"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Cryptocurrency Wallet SMO

cryptocurrency_wallet_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"cryptocurrency-wallet")), # extension-definition--be78509e-6958-51b1-8b26-d17ee0eba2d7
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2020, 1, 1, tzinfo=UTC),
                        name="Cryptocurrency Wallet",
                        description="This extension creates a new SCO that can be used to represent cryptocurrency wallets.",
                        schema=schema_base+"scos/cryptocurrency-wallet.json",
                        version="1.0",
                        extension_types=[
                            "new-sco"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Bank Account SMO

bank_account_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"bank-account")), # extension-definition--f19f3291-6a84-5674-b311-d75a925d5bd9
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2020, 1, 1, tzinfo=UTC),
                        name="Bank Account",
                        description="This extension creates a new SCO that can be used to represent bank account details.",
                        schema=schema_base+"scos/bank-account.json",
                        version="1.0",
                        extension_types=[
                            "new-sco"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Vulnerability Scoring Extension SMO

vulnerability_scoring_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"vulnerability-scoring")), # extension-definition--2c5c13af-ee92-5246-9ba7-0b958f8cd34a
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2025, 10, 3, 0, 0, tzinfo=UTC),
                        name="Vulnerability SDO Scoring Properties",
                        description="This extension adds new properties to Vulnerbility SDOs to provide scoring.",
                        schema=schema_base+"properties/vulnerability-scoring.json",
                        version="1.0",
                        extension_types=[
                            "toplevel-property-extension"
                        ],
                        extension_properties=[
                            "x_cvss",
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Vulnerability OpenCTI Extension SMO

vulnerability_opencti_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"vulnerability-opencti")), # xtension-definition--ec658473-1319-53b4-879f-488e47805554
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=modified,
                        name="Vulnerability SDO OpenCTI Properties",
                        description="This extension adds OpenCTI-specific properties to STIX Vulnerability SDOs, including CVSS v2/v3/v4 metrics, CWE, CISA KEV, and EPSS.",
                        schema=schema_base+"properties/vulnerability-opencti.json",
                        version="1.0",
                        extension_types=[
                            "toplevel-property-extension"
                        ],
                        extension_properties=[
                            "x_opencti_cvss_v2_base_score",
                            "x_opencti_cvss_v2_temporal_score",
                            "x_opencti_cvss_v2_vector_string",
                            "x_opencti_cvss_v2_access_vector",
                            "x_opencti_cvss_v2_access_complexity",
                            "x_opencti_cvss_v2_authentication",
                            "x_opencti_cvss_v2_confidentiality_impact",
                            "x_opencti_cvss_v2_integrity_impact",
                            "x_opencti_cvss_v2_availability_impact",
                            "x_opencti_cvss_v2_exploitability",
                            "x_opencti_cvss_v2_remediation_level",
                            "x_opencti_cvss_v2_report_confidence",
                            "x_opencti_cvss_base_score",
                            "x_opencti_cvss_temporal_score",
                            "x_opencti_cvss_vector_string",
                            "x_opencti_cvss_base_severity",
                            "x_opencti_cvss_attack_vector",
                            "x_opencti_cvss_attack_complexity",
                            "x_opencti_cvss_privileges_required",
                            "x_opencti_cvss_user_interaction",
                            "x_opencti_cvss_scope",
                            "x_opencti_cvss_confidentiality_impact",
                            "x_opencti_cvss_integrity_impact",
                            "x_opencti_cvss_availability_impact",
                            "x_opencti_cvss_exploit_code_maturity",
                            "x_opencti_cvss_remediation_level",
                            "x_opencti_cvss_report_confidence",
                            "x_opencti_cvss_v4_base_score",
                            "x_opencti_cvss_v4_vector_string",
                            "x_opencti_cvss_v4_base_severity",
                            "x_opencti_cvss_v4_attack_vector",
                            "x_opencti_cvss_v4_attack_complexity",
                            "x_opencti_cvss_v4_attack_requirements",
                            "x_opencti_cvss_v4_privileges_required",
                            "x_opencti_cvss_v4_user_interaction",
                            "x_opencti_cvss_v4_confidentiality_impact_v",
                            "x_opencti_cvss_v4_confidentiality_impact_s",
                            "x_opencti_cvss_v4_integrity_impact_v",
                            "x_opencti_cvss_v4_integrity_impact_s",
                            "x_opencti_cvss_v4_availability_impact_v",
                            "x_opencti_cvss_v4_availability_impact_s",
                            "x_opencti_cvss_v4_exploit_maturity",
                            "x_opencti_cwe",
                            "x_opencti_cisa_kev",
                            "x_opencti_epss_score",
                            "x_opencti_epss_percentile"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Location OpenCTI Extension SMO

location_opencti_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"location-opencti")), # extension-definition--b9c1f945-80be-519d-9d7f-0cede26032e9
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=modified,
                        name="Location SDO OpenCTI Properties",
                        description="This extension adds OpenCTI-specific properties to STIX Location SDOs.",
                        schema=schema_base+"properties/location-opencti.json",
                        version="1.0",
                        extension_types=[
                            "toplevel-property-extension"
                        ],
                        extension_properties=[
                            "x_opencti_aliases",
                            "x_opencti_location_type"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Indicator Vulnerable CPEs Extension SMO

indicator_vulnerable_cpes_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"indicator-vulnerable-cpes")), # extension-definition--ad995824-2901-5f6e-890b-561130a239d4
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2020, 1, 1, tzinfo=UTC),
                        name="Indicator SDO Vulnerable CPEs Properties",
                        description="This extension adds new properties to Indicator SDOs to list CPE vulnerable inside a pattern.",
                        schema=schema_base+"properties/indicator-vulnerable-cpes.json",
                        version="1.0",
                        extension_types=[
                            "toplevel-property-extension"
                        ],
                        extension_properties=[
                            "x_cpes"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Report EPSS Scoring SMO

report_epss_scoring_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"report-epss-scoring")), # extension-definition--f80cce10-5ac0-58d1-9e7e-b4ed0cc4dbb9
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2020, 1, 1, tzinfo=UTC),
                        name="Report SDO EPSS Scoring Properties",
                        description="This extension adds new properties to Report SDOs to capture EPSS scores for CVEs.",
                        schema=schema_base+"properties/report-epss-scoring.json",
                        version="1.0",
                        extension_types=[
                            "toplevel-property-extension"
                        ],
                        extension_properties=[
                            "x_epss"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Software CPE SMO

software_cpe_properties_ExtensionDefinitionSMO = ExtensionDefinition(
                        id="extension-definition--" + str(uuid.uuid5(namespace, f"software-cpe-properties")), # extension-definition--82cad0bb-0906-5885-95cc-cafe5ee0a500
                        created_by_ref=DOGESEC_IDENTITY_REF,
                        created=created,
                        modified=datetime(2025, 10, 3, tzinfo=UTC),
                        name="Software SCO CPE Properties",
                        description="This extension adds new properties to Software SCOs to capture CPE data.",
                        schema=schema_base+"properties/software-cpe-properties.json",
                        version="1.0",
                        extension_types=[
                            "toplevel-property-extension"
                        ],
                        extension_properties=[
                            "x_cpe_struct",
                            "x_revoked",
                            "x_created",
                            "x_modified"
                        ],
                        object_marking_refs=object_marking_refs
                    )

### Attack Flow SMO

attack_flow_ExtensionDefinitionSMO = ExtensionDefinition(
                        type="extension-definition",
                        id="extension-definition--fb9c968a-745b-4ade-9b25-c324172197f4",
                        spec_version="2.1",
                        created="2022-08-02T19:34:35.143Z",
                        modified="2022-08-02T19:34:35.143Z",
                        name="Attack Flow",
                        description="Extends STIX 2.1 with features to create Attack Flows.",
                        created_by_ref="identity--fb9c968a-745b-4ade-9b25-c324172197f4",
                        schema="https://center-for-threat-informed-defense.github.io/attack-flow/stix/attack-flow-schema-2.0.0.json",
                        version="2.0.0",
                        extension_types=["new-sdo"],
                        external_references=[
                            {
                                "source_name": "Documentation",
                                "description": "Documentation for Attack Flow",
                                "url": "https://center-for-threat-informed-defense.github.io/attack-flow",
                            },
                            {
                                "source_name": "GitHub",
                                "description": "Source code repository for Attack Flow",
                                "url": "https://github.com/center-for-threat-informed-defense/attack-flow",
                            },
                        ],
                    )




from stix2 import CustomObservable
from stix2.properties import (
    ExtensionsProperty, ReferenceProperty,
    IDProperty, ListProperty, StringProperty, BooleanProperty,
    TypeProperty
)
from ._extensions import bank_card_ExtensionDefinitionSMO

_type = 'bank-card'
@CustomObservable('bank-card', [
    ('type', TypeProperty(_type, spec_version='2.1')),
    ('spec_version', StringProperty(fixed='2.1')),
    ('id', IDProperty(_type, spec_version='2.1')),
    ('format', StringProperty()),
    ('number', StringProperty(required=True)),
    ('scheme', StringProperty()),
    ('brand', StringProperty()),
    ('currency', StringProperty()),
    ('issuer_ref', ReferenceProperty(valid_types='identity', spec_version='2.1')),
    ('holder_ref', ReferenceProperty(valid_types='identity', spec_version='2.1')),
    ('valid_from', StringProperty()),
    ('valid_to', StringProperty()),
    ('security_code', StringProperty()),
    ('level', StringProperty()),
    ('is_commercial', BooleanProperty()),
    ('is_prepaid', BooleanProperty()),
    # ('extensions', ExtensionsProperty(spec_version='2.1'))
], extension_name=bank_card_ExtensionDefinitionSMO.id, id_contrib_props=['number'])
class BankCard(object):
    pass



from stix2 import CustomObject
from stix2.properties import (
    ExtensionsProperty, ReferenceProperty,
    IDProperty, ListProperty, StringProperty,
    TypeProperty,
)

from ._extensions import attack_flow_ExtensionDefinitionSMO


_type = 'attack-flow'
@CustomObject(_type, [
    ('type', TypeProperty(_type, spec_version='2.1')),
    ('spec_version', StringProperty(fixed='2.1')),
    ('id', IDProperty(_type, spec_version='2.1')),
    ('name', StringProperty(required=True)),
    ('description', StringProperty()),
    ('scope', StringProperty(required=True)),
    ('start_refs', ListProperty(ReferenceProperty(valid_types=['attack-action']))),
    
], extension_name=attack_flow_ExtensionDefinitionSMO.id)
class AttackFlow(object):
    pass


_type = 'attack-action'
@CustomObject(_type, [
    ('type', TypeProperty(_type, spec_version='2.1')),
    ('spec_version', StringProperty(fixed='2.1')),
    ('id', IDProperty(_type, spec_version='2.1')),
    ('technique_id', StringProperty(required=True)),
    ('technique_ref', ReferenceProperty(required=True, valid_types=['attack-pattern', 'x-mitre-tactic'])),
    ('tactic_id', StringProperty(required=True)),
    ('tactic_ref', ReferenceProperty(required=True, valid_types=['x-mitre-tactic'])),
    ('name', StringProperty(required=True)),
    ('effect_refs',  ListProperty(ReferenceProperty(valid_types=['attack-action']))),
])
class AttackAction(object):
    with_extension = AttackFlow.with_extension



import uuid
import stix2
import os
import shutil

from uuid import UUID
from stix2extensions.cryptocurrency_transaction import CryptocurrencyTransaction
# create the directories

tmp_directories = [
    "tmp_object_store",
]

for directory in tmp_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

# define UUID for generating UUIDv5s -- this is the OASIS namespace for SCOs https://github.com/oasis-open/cti-python-stix2/blob/master/stix2/base.py#L29

namespace=UUID("00abedb4-aa42-466c-9c01-fed23315a9b7")

# Create CryptocurrencyTransaction SCO

example_CryptocurrencyTransactionSCO = CryptocurrencyTransaction(
                    id="cryptocurrency-transaction--"+ str(uuid.uuid5(namespace, f"3FZbgi29cpjq2GjdwV8eyHuJJnkLtktZc5")), # cryptocurrency-transaction--da6911a9-7f49-5d20-ae4e-f9154d1b9a39
                    symbol="BTC",
                    hash="3FZbgi29cpjq2GjdwV8eyHuJJnkLtktZc5",
                    fee="0.00000728",
                    execution_time="2022-10-02T15:22:21Z",
                    input=[
                        {
                            "address_ref": "cryptocurrency-wallet--b1bc02bd-f10d-40e4-86d1-9e2ef3aa919f",
                            "amount": 0.84000000
                        }
                    ],
                    output=[
                        {
                            "address_ref": "cryptocurrency-wallet--b6ae6083-875f-5fe9-84b9-89e168f89406",
                            "amount": 0.80000000
                        },
                        {
                            "address_ref": "cryptocurrency-wallet--4d18102a-f35a-4176-9509-f96a3370d067",
                            "amount": 0.04000000
                        }
                    ],
                    )

# Write the objects to the filestore
## https://stix2.readthedocs.io/en/latest/guide/filesystem.html#FileSystemSource

### Creating FileSystemStore and adding MarkingDefinitionSMO for each directory

fs_directories = {
    "tmp_object_store": example_CryptocurrencyTransactionSCO
}

for directory, cryptocurrency_transaction_sco in fs_directories.items():
    fs_store = stix2.FileSystemStore(directory)
    fs_store.add([cryptocurrency_transaction_sco])

# Now move those files into the standardised locations for easy download

final_directories = [
    "example_objects/scos"
]

for directory in final_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

shutil.move("tmp_object_store/cryptocurrency-transaction/cryptocurrency-transaction--" + str(uuid.uuid5(namespace, f"3FZbgi29cpjq2GjdwV8eyHuJJnkLtktZc5")) + ".json", "example_objects/scos/cryptocurrency-transaction--" + str(uuid.uuid5(namespace, f"3FZbgi29cpjq2GjdwV8eyHuJJnkLtktZc5")) + ".json")

shutil.rmtree("tmp_object_store")


import uuid
import stix2
import os
import shutil

from uuid import UUID

from stix2extensions.phone_number import Phonenumber
# create the directories

tmp_directories = [
    "tmp_object_store",
]

for directory in tmp_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

# define UUID for generating UUIDv5s -- this is the OASIS namespace for SCOs https://github.com/oasis-open/cti-python-stix2/blob/master/stix2/base.py#L29

namespace=UUID("00abedb4-aa42-466c-9c01-fed23315a9b7")

# Create PhoneNumber SCO

example_PhoneNumberSCO = Phonenumber(
                    id="phone-number--"+ str(uuid.uuid5(namespace, f"07890129093")),
                    number="4407890129093",
                    country="GBR",
                    connection="mobile",
                    provider="Big Network"
                    )

# Write the objects to the filestore
## https://stix2.readthedocs.io/en/latest/guide/filesystem.html#FileSystemSource

### Creating FileSystemStore and adding MarkingDefinitionSMO for each directory

fs_directories = {
    "tmp_object_store": example_PhoneNumberSCO
}

for directory, phone_number_sco in fs_directories.items():
    fs_store = stix2.FileSystemStore(directory)
    fs_store.add([phone_number_sco])

# Now move those files into the standardised locations for easy download

final_directories = [
    "example_objects/scos"
]

for directory in final_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

shutil.move("tmp_object_store/phone-number/phone-number--" + str(uuid.uuid5(namespace, f"07890129093")) + ".json", "example_objects/scos/phone-number--" + str(uuid.uuid5(namespace, f"07890129093")) + ".json")

shutil.rmtree("tmp_object_store")


import uuid
import stix2
import os
import shutil

from uuid import UUID

from stix2 import Software
from stix2extensions._extensions import software_cpe_properties_ExtensionDefinitionSMO

# create the directories

tmp_directories = [
    "tmp_object_store",
]

for directory in tmp_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

# Create SoftwareSCO object

### software--fda5adb5-23e5-5089-a256-36f298ba241f

example_SoftwareSCO = Software(
                        id="software--fda5adb5-23e5-5089-a256-36f298ba241f",
                        name="EGroupware 14.1.20140710 Community Edition",
                        cpe="cpe:2.3:a:egroupware:egroupware:14.1.20140710:*:*:*:community:*:*:*",
                        swid="A1F2EAFC-0523-4257-A9EA-94462CA2BDB8",
                        languages=[
                            "en"
                        ],
                        vendor="egroupware",
                        version="14.1.20140710",
                        x_cpe_struct={
                            "cpe_version": "2.3",
                            "part": "a",
                            "vendor": "egroupware",
                            "product": "egroupware",
                            "version": "14.1.20140710",
                            "update": "*",
                            "edition": "*",
                            "language": "*",
                            "sw_edition": "community",
                            "target_sw": "*",
                            "target_hw": "*",
                            "other": "*"
                        },
                        x_revoked=False,
                        x_created="2020-01-01T00:00:00.000Z",
                        x_modified="2020-01-01T00:00:00.000Z",
                        extensions={
                            software_cpe_properties_ExtensionDefinitionSMO.id: {
                                    "extension_type": "toplevel-property-extension"
                            }
                        }
                    )


# Write the objects to the filestore
## https://stix2.readthedocs.io/en/latest/guide/filesystem.html#FileSystemSource

### Creating FileSystemStore and adding MarkingDefinitionSMO for each directory

fs_directories = {
    "tmp_object_store": example_SoftwareSCO
}

for directory, note_sdo in fs_directories.items():
    fs_store = stix2.FileSystemStore(directory)
    fs_store.add([note_sdo])

# Now move those files into the standardised locations for easy download

final_directories = [
    "example_objects/scos"
]

for directory in final_directories:
    if not os.path.exists(directory):
        os.makedirs(directory)

shutil.move("tmp_object_store/software/software--fda5adb5-23e5-5089-a256-36f298ba241f.json", "example_objects/properties/software--fda5adb5-23e5-5089-a256-36f298ba241f.json")

shutil.rmtree("tmp_object_store")